import ceasylog.LoggerLevel as LoggerLevel


class LoggerConfiger(object):

    def __init__(self):
        self.__name = "default"

        self.__maxPrintLevel = LoggerLevel.CRITICAL
        self.__minPrintLevel = LoggerLevel.DEBUG
        self.__maxRecordLevel = LoggerLevel.CRITICAL
        self.__minRecordLevel = LoggerLevel.WARN

        self.__printTimeFormat = "%Y-%m-%d %H:%M:%S.%f"
        self.__recordTimeFormat = "%Y-%m-%d %H:%M:%S.%f"
        self.__recordPathNameFormat = "%Y-%m-%d"

        self.__isRecordB = False
        self.__recordPath = "./"

    @property
    def name(self):
        return self.__name

    @property
    def maxPrintLevel(self):
        return self.__maxPrintLevel

    @property
    def minPrintLevel(self):
        return self.__minPrintLevel

    @property
    def maxRecordLevel(self):
        return self.__maxRecordLevel

    @property
    def minRecordLevel(self):
        return self.__minRecordLevel

    @property
    def printTimeFormat(self):
        return self.__printTimeFormat

    @property
    def recordTimeFormat(self):
        return self.__recordTimeFormat

    @property
    def recordPathNameFormat(self):
        return self.__recordPathNameFormat

    @property
    def isRecordB(self):
        return self.__isRecordB

    @property
    def recordPath(self):
        return self.__recordPath

    def loadFromFile(self, path: str):
        import os
        import json
        if not os.path.isabs(path):
            # 拼接解释器运行路径作为绝对路径
            path = os.path.join(os.path.dirname(os.path.abspath(__file__)), path)
        if not os.path.exists(path):
            raise FileNotFoundError(f"Could not find LoggerConfiger config file: {path}")
        try:
            with open(path, "r", encoding="utf-8") as fileObject:
                fileContent: str = fileObject.read()
            configObject: dict = json.loads(fileContent)
        except Exception as e:
            raise Exception(f"Could not read LoggerConfiger config file: {path} because {str(e)}")
        for key in configObject.keys():
            value = configObject[key]
            if key == "name":
                self.setName(value)
            elif key == "maxPrintLevel":
                if value == "CRITICAL":
                    value = LoggerLevel.CRITICAL
                elif value == "ERROR":
                    value = LoggerLevel.ERROR
                elif value == "WARN":
                    value = LoggerLevel.WARN
                elif value == "INFO":
                    value = LoggerLevel.INFO
                elif value == "DEBUG":
                    value = LoggerLevel.DEBUG
                else:
                    raise ValueError(f"Paser LoggerConfiger failed: Unknown value {value} for key maxPrintLevel")
                self.setMaxPrintLevel(value)
            elif key == "minPrintLevel":
                if value == "CRITICAL":
                    value = LoggerLevel.CRITICAL
                elif value == "ERROR":
                    value = LoggerLevel.ERROR
                elif value == "WARN":
                    value = LoggerLevel.WARN
                elif value == "INFO":
                    value = LoggerLevel.INFO
                elif value == "DEBUG":
                    value = LoggerLevel.DEBUG
                else:
                    raise ValueError(f"Paser LoggerConfiger failed: Unknown value {value} for key minPrintLevel")
                self.setMinPrintLevel(value)
            elif key == "maxRecordLevel":
                if value == "CRITICAL":
                    value = LoggerLevel.CRITICAL
                elif value == "ERROR":
                    value = LoggerLevel.ERROR
                elif value == "WARN":
                    value = LoggerLevel.WARN
                elif value == "INFO":
                    value = LoggerLevel.INFO
                elif value == "DEBUG":
                    value = LoggerLevel.DEBUG
                else:
                    raise ValueError(f"Paser LoggerConfiger failed: Unknown value {value} for key maxRecordLevel")
                self.setMaxRecordLevel(value)
            elif key == "minRecordLevel":
                if value == "CRITICAL":
                    value = LoggerLevel.CRITICAL
                elif value == "ERROR":
                    value = LoggerLevel.ERROR
                elif value == "WARN":
                    value = LoggerLevel.WARN
                elif value == "INFO":
                    value = LoggerLevel.INFO
                elif value == "DEBUG":
                    value = LoggerLevel.DEBUG
                else:
                    raise ValueError(f"Paser LoggerConfiger failed: Unknown value {value} for key minPrintLevel")
                self.setMinRecordLevel(value)
            elif key == "printTimeFormat":
                self.setPrintTimeFormat(value)
            elif key == "recordTimeFormat":
                self.setRecordTimeFormat(value)
            elif key == "recordPathNameFormat":
                self.setRecordPathNameFormat(value)
            elif key == "isRecord":
                self.isRecord(value)
            else:
                raise KeyError(f"Paser LoggerConfiger failed: Unknown key {key}")

    def saveToFile(self, path: str):
        import os
        import json
        # 存储为配置文件的内容
        if self.isRecordB:
            configObject = {
                "name": self.name,
                "maxPrintLevel": self.maxPrintLevel.des,
                "minPrintLevel": self.minPrintLevel.des,
                "maxRecordLevel": self.maxRecordLevel.des,
                "minRecordLevel": self.minRecordLevel.des,
                "printTimeFormat": self.printTimeFormat,
                "recordTimeFormat": self.recordTimeFormat,
                "recordPathNameFormat": self.recordPathNameFormat,
                "isRecord": self.recordPath
            }
        else:
            configObject = {
                "name": self.name,
                "maxPrintLevel": self.maxPrintLevel.des,
                "minPrintLevel": self.minPrintLevel.des,
                "maxRecordLevel": self.maxRecordLevel.des,
                "minRecordLevel": self.minRecordLevel.des,
                "printTimeFormat": self.printTimeFormat,
            }
        configContent: str = json.dumps(configObject, indent=4, ensure_ascii=False)
        if not os.path.isabs(path):
            # 拼接解释器运行路径作为绝对路径
            path = os.path.join(os.path.dirname(os.path.abspath(__file__)), path)
        if not os.path.exists(os.path.dirname(path)):
            os.makedirs(os.path.dirname(path))
        if os.path.exists(path):
            os.remove(path)
        try:
            with open(path, "w", encoding="utf-8") as fileObject:
                fileObject.write(configContent)
        except OSError as e:
            raise OSError(f"Could not save LoggerConfiger config file: {path} because {str(e)}")

    def extendBy(self, father):
        if isinstance(father, LoggerConfiger):
            self.__name = f"default (extends by {father.name})"

            self.__maxPrintLevel = father.maxPrintLevel
            self.__minPrintLevel = father.minPrintLevel
            self.__maxRecordLevel = father.maxRecordLevel
            self.__minRecordLevel = father.minRecordLevel

            self.__printTimeFormat = father.printTimeFormat
            self.__recordTimeFormat = father.recordTimeFormat
            self.__recordPathNameFormat = father.recordPathNameFormat

            self.__isRecordB = father.isRecordB
            self.__recordPath = father.recordPath

    def setName(self, name: str):
        self.__name = name.strip()

    def setMaxPrintLevel(self, maxPrintLevel):
        self.__maxPrintLevel = maxPrintLevel

    def setMinPrintLevel(self, minPrintLevel):
        self.__minPrintLevel = minPrintLevel

    def setMaxRecordLevel(self, maxRecordLevel):
        self.__maxRecordLevel = maxRecordLevel

    def setMinRecordLevel(self, minRecordLevel):
        self.__minRecordLevel = minRecordLevel

    def setPrintTimeFormat(self, printTimeFormat: str):
        self.__printTimeFormat = printTimeFormat

    def setRecordTimeFormat(self, recordTimeFormat: str):
        self.__recordTimeFormat = recordTimeFormat

    def setRecordPathNameFormat(self, printTimeFormat: str):
        self.__printTimeFormat = printTimeFormat

    def isRecord(self, path: str):
        self.__isRecordB = True
        self.__recordPath = path

    def setRecordPath(self, recordPath: str):
        self.__recordPath = recordPath



from colorama import Fore, Back


class DEBUG:
    des = "DEBUG"
    level = 1
    msg = " D "
    frontStyle = Fore.BLUE
    backStyle = Back.BLUE + Fore.LIGHTWHITE_EX


class INFO:
    des = "INFO"
    level = 2
    msg = " I "
    frontStyle = Fore.GREEN
    backStyle = Back.GREEN + Fore.LIGHTWHITE_EX


class WARN:
    des = "WARN"
    level = 3
    msg = " W "
    frontStyle = Fore.YELLOW
    backStyle = Back.YELLOW + Fore.LIGHTWHITE_EX


class ERROR:
    des = "ERROR"
    level = 4
    msg = " E "
    frontStyle = Fore.RED
    backStyle = Back.RED + Fore.LIGHTWHITE_EX


class CRITICAL:
    des = "CRITICAL"
    level = 5
    msg = " C "
    frontStyle = Fore.LIGHTRED_EX
    backStyle = Back.LIGHTRED_EX + Fore.LIGHTWHITE_EX



from setuptools import setup, find_packages

setup(
    name='ceasylog',
    version='1.1.1',
    keywords='ceasylog',
    description='A great log util from CandyStar',
    author='CandyStar@HuangXudong',
    license='Apache License 2.0',
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    install_requires=['colorama'],
)



import os
import platform
import random
import string
import uuid
from datetime import datetime

from colorama import Fore, Style
from colorama import init as coloramaInit

from ceasylog.LoggerConfiger import *

coloramaInit(autoreset=True)

# Surprise Mother F**K LMX

LOG_STYLE = [
    Fore.LIGHTYELLOW_EX,
    Fore.LIGHTBLUE_EX,
    Fore.LIGHTCYAN_EX,
    Fore.LIGHTGREEN_EX,
    Fore.LIGHTMAGENTA_EX,
]

MAX_BLANK = 35


def getTime(timeFormat: str) -> str:
    """
    获取时间
    :param timeFormat: 时间格式
    :return:
    """
    return datetime.now().strftime(timeFormat)


def getSysFileSplit():
    """
    获取操作系统的文件分隔符
    :return:
    """
    if platform.system() == "Windows":
        return "\\"
    else:
        return "/"


def getUID():
    """
    生成唯一标识符
    :return:
    """
    randomString = ''.join(random.choices(string.ascii_letters + string.digits, k=4))
    uniqueId = str(uuid.uuid4())[:6] + randomString
    return uniqueId


def logPrint(msg: str, level: LoggerLevel, config: LoggerConfiger, uid: str, logStyle):
    """
    打印日志
    :param msg: 信息
    :param level: 等级
    :param config: 配置
    :param uid: 唯一标识符
    :param logStyle: logger名称样式
    :return:
    """
    if level.level < config.minPrintLevel.level or level.level > config.maxPrintLevel.level:
        return
    nowTime = getTime(config.printTimeFormat)
    blank = " " * (MAX_BLANK - len(config.name))
    logContent = (f"{Fore.GREEN}{nowTime} {Fore.MAGENTA}{uid} {logStyle}{config.name}{Style.RESET_ALL}{blank} "
                  f"{level.backStyle} {level.msg} {Style.RESET_ALL} {level.frontStyle} {msg}")
    print(logContent)


def logRecord(msg: str, level: LoggerLevel, config: LoggerConfiger, uid: str):
    """
    记录日志到文件
    :param msg: 信息
    :param level: 等级
    :param config: 配置
    :param uid: 唯一标识符
    :return:
    """
    if level.level < config.minRecordLevel.level or level.level > config.maxRecordLevel.level or not config.isRecordB:
        return
    nowTime = getTime(config.recordTimeFormat)
    blank = " " * (MAX_BLANK - len(config.name))
    logContent = f"{nowTime} {uid} {config.name}{blank} {level.msg} {msg}"
    try:
        if not config.recordPath.endswith(getSysFileSplit()):
            recordPath = config.recordPath + getSysFileSplit()
        else:
            recordPath = config.recordPath
        recordFullFileName = recordPath + getTime(config.recordPathNameFormat) + ".log"
        if not os.path.exists(os.path.dirname(recordFullFileName)):
            os.makedirs(os.path.dirname(recordFullFileName))
        with open(recordFullFileName, "a", encoding="utf-8") as logRecordFileObject:
            logRecordFileObject.write(logContent + "\n")
    except Exception as e:
        easyLogBuildInLogger.error("Failed to write log file " + str(e))


class Logger(object):
    """
    日志记录器
    """
    def __init__(self, config: LoggerConfiger):
        """
        :param config: 配置信息
        """
        self.config = config
        # 超过长度的名称省略
        if len(self.config.name) > MAX_BLANK:
            self.config.setName(self.config.name[:(MAX_BLANK - 13)] + "..." + self.config.name[-10:])
        self.logStyle = LOG_STYLE[random.randint(0, len(LOG_STYLE) - 1)]

    def debug(self, msg: str):
        uid = getUID()
        logPrint(msg, LoggerLevel.DEBUG, self.config, uid, self.logStyle)
        logRecord(msg, LoggerLevel.DEBUG, self.config, uid)

    def info(self, msg: str):
        uid = getUID()
        logPrint(msg, LoggerLevel.INFO, self.config, uid, self.logStyle)
        logRecord(msg, LoggerLevel.INFO, self.config, uid)

    def warn(self, msg: str):
        uid = getUID()
        logPrint(msg, LoggerLevel.WARN, self.config, uid, self.logStyle)
        logRecord(msg, LoggerLevel.WARN, self.config, uid)

    def error(self, msg: str):
        uid = getUID()
        logPrint(msg, LoggerLevel.ERROR, self.config, uid, self.logStyle)
        logRecord(msg, LoggerLevel.ERROR, self.config, uid)

    def critical(self, msg: str):
        uid = getUID()
        logPrint(msg, LoggerLevel.CRITICAL, self.config, uid, self.logStyle)
        logRecord(msg, LoggerLevel.CRITICAL, self.config, uid)

    # def d(self, msg):
    #     self.debug(msg)
    #
    # def i(self, msg):
    #     self.info(msg)
    #
    # def w(self, msg):
    #     self.warn(msg)
    #
    # def e(self, msg):
    #     self.error(msg)
    #
    # def c(self, msg: str):
    #     self.critical(msg)


easyLogBuildInLoggerCfg = LoggerConfiger()
easyLogBuildInLoggerCfg.setName("easyLog_BuildInLogger")
easyLogBuildInLoggerCfg.setMinPrintLevel(LoggerLevel.WARN)
easyLogBuildInLogger = Logger(easyLogBuildInLoggerCfg)
